package org.illeva.jordino.core;

import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import org.illeva.jordino.answer.*;

/**
  * This thread empty the park zone to answer incoming messages.
  *
  * @version Jordino V1.0
  * @author IlLeva
  */ 
public class MailReader extends Thread
{
  // Data
  /** If true, the thread is running. */
  private boolean dSemaphore=false;
  /** Debug status. */
  private boolean dDebug=false;
  /** Park zone for incoming messages */
  private MessageQueue dQueue;
  /** host for send answer. */
  private String dSmtp="localhost";
  /** Answer's factory */
  private AnswerFactory dFactory=new AnswerFactory();
  // Methods
  /**
    * Build a responder object.
    * @param aQueue A queue of messages to respond.
    * @since Jordino Version 1.0
    */
  public MailReader(MessageQueue aQueue)
  {
    dQueue=aQueue;
    // Read properties file.
    Properties vProp=new Properties();
    try
    {
      vProp.load(new FileInputStream("../conf/jordino.properties"));
      dSmtp=vProp.getProperty("smtp.url","localhost");
    } // end try
    catch(IOException anEx)
    {
      System.err.println(anEx);
    } // end catch
  } // end constructor
  /**
    * This method returns the state of semaphore to control the thread.
    * @since Jordino Version 1.0
    */
  synchronized public boolean getSemaphore() {return dSemaphore;}
  /**
    * This method sets the state of semaphore to control the thread.
    * @since Jordino Version 1.0
    */
  synchronized public void setSemaphore(boolean aFlag) {dSemaphore=aFlag;}
  /**
    * Thread at work. To control the thread status, use the
    * setSemaphore() function.
    * @since Jordino Version 1.0
    */
  public void run()
  {
    System.out.println("MailReader Run...");
    Properties vProp= System.getProperties();
    vProp.put("mail.smtp.host",dSmtp);
    Session vSession= Session.getDefaultInstance(vProp, null);
    System.out.println("MailReader started.");
    while(getSemaphore())
    {
      Message vMsg=dFactory.run(dQueue.pop());
      if(vMsg!=null)
        try
        {
          vMsg.setSentDate(new Date());
          Transport.send(vMsg);
        } // end try
        catch(MessagingException anEx2)
        {
          System.err.println(anEx2);
        } // end catch
    } // end while
    System.out.println("MailReader stopped.");
  } // end run
} // end class