package org.illeva.jordino.core;

import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

/**
  * This is a FIFO queue to park the incoming messages.
  *
  * @version Jordino V1.0
  * @author IlLeva
  */
public class MessageQueue 
{
    // Data
    /** Where messages are stored */
    private LinkedList dBuffer=new LinkedList();
    // methods
    /**
      * Build a park zone.
      */
    public MessageQueue()
    {
    } // end constructor
    /**
      * Return an incoming message to process.<BR>
      * This method stop the ReaderMail thread until there is a message
      * to process.
      * @since Jordino Version 1.0
      */
  synchronized public MimeMessage pop()
  {
    boolean vForceExit=false;
    MimeMessage vRc=null;
    while(!vForceExit && dBuffer.isEmpty())
    {
      try
      {
        System.out.println("Sto aspettando nel pop()");
        wait();
      }
      catch(InterruptedException anEx)
      {
        vForceExit=true;
      } // end catch
    } // end while
    notifyAll();
    if(!dBuffer.isEmpty())
      vRc=(MimeMessage)dBuffer.removeLast();
    return vRc;
  } // end pop
  /**
    * Populate the park zone with incoming messages.
    * Only the messages with string "[jd" into subject are elaborated end
    * deleted from inbox.
    * Notify all the event.
    * @since Jordino Version 1.0
    */

  synchronized public void push(Message aMsg)
  {
    /* Flag per l'interruzione del programma
    da parte dell'utente. */
    boolean vForceExit=false;
    while(!vForceExit && !dBuffer.isEmpty())
    {
      /* Entra nel ciclo se ci sono messaggi da processare.
      Aspetta che vengano processati. */
      try
      {
        System.out.println("Sto aspettando nel push()");
        wait();
      } // end try
      catch(InterruptedException anEx)
      {
        vForceExit=true;
      } // end catch
    } // end while
    try
    {
      /* Se tutti i messaggi nel buffer sono stati processati, aggiunge un nuovo 
      messaggio nel buffer e sveglia l'altro thread. */
      if(aMsg.getSubject().indexOf("[JD")!=-1)
      {
        dBuffer.addFirst(aMsg);
        aMsg.setFlag(Flags.Flag.DELETED,true);
        notifyAll();
      } // end if
    } // end try
    catch(MessagingException anEx)
    {
      System.err.println(anEx);
    } // end catch
  } // end push
} // end class