package org.illeva.jordino.core;

import java.io.*;
import java.util.*;
import javax.mail.*;

/**
  * This thread listen the pop look for incoming messages.
  *
  * @version Jordino V1.0
  * @author IlLeva
  */
public class WatchDog extends Thread
{
  // Data
  /** This is the flag to control the state of thread. */
  private boolean dSemaphore=false;
  /** host to control */
  private String dPop="localhost";
  /** Password to access the pop */
  private String dPassword="password";
  /** User to acces the pop */
  private String dUser="user";
  /** Folder to check */
  private String dFolderName="inbox";
  /** Sleeping time beetween two contiguos checks. */
  private int dSleepSecs=60;
  /** Debug control */
  private boolean dDebug=false;
  /** Object's status */
  private int dStatus=0;
  /** Par zone  for incomin mesages. */
  private MessageQueue dQueue=new MessageQueue();
  // Methods
  /**
    * Builds a Watchdog object.
    * @since  Jordino Version 1.0
    */
  public WatchDog(MessageQueue aQueue)
  {
    dQueue=aQueue;
    // Read properties file.
    Properties vProp=new Properties();
    try
    {
      vProp.load(new FileInputStream("../conf/jordino.properties"));
      dPop=vProp.getProperty("pop3.url","localhost");
      dPassword=vProp.getProperty("pop3.password","password");
      dUser=vProp.getProperty("pop3.user","user");
      dFolderName=vProp.getProperty("pop3.inbox.name","inbox");
      dSleepSecs=Integer.parseInt(vProp.getProperty("sleep.sec.time","60"));
    } // end try
    catch(IOException anEx)
    {
      System.err.println(anEx);
    } // end catch
  } // end constructor
  /**
    * Return the status of object.
    * @since  Jordino Version 1.0
    */
  public int getStatus() {return dStatus;}
  /**
    * Return a boolean to indicate if the thread is running.
    * @since  Jordino Version 1.0
    */
  synchronized public boolean getSemaphore() {return dSemaphore;}
  /**
    * Set the boolean flag to start or stop the thread.
    * @since  Jordino Version 1.0
    */
  synchronized public void setSemaphore(boolean aFlag) {dSemaphore=aFlag;}
  /**
    * Thread at work.
    * @since  Jordino Version 1.0
    */
  public void run()
  {
    System.out.println("WatchDog Run...");
    dStatus=0;
    Properties vProp=System.getProperties();
    Session vSession = Session.getDefaultInstance(vProp,null);
    if(dDebug)
      vSession.setDebug(true);
    try
    {
      // Store vStore = vSession.getStore("imap");
      Store vStore = vSession.getStore("pop3");
      vStore.connect(dPop,dUser,dPassword);
      Folder vFolder = vStore.getFolder(dFolderName);
      if(vFolder!=null && vFolder.exists())
      {
        // vFolder.addMessageCountListener(new Dog(dQueue));
        System.out.println("WatchDoc active on "+dPop);
        while(getSemaphore())
        {
          vFolder.open(Folder.READ_WRITE);
          Message[] vMsgs=vFolder.getMessages();
          for(int i=0; i<vMsgs.length; i++)
          {
              dQueue.push(vMsgs[i]);
          } // end for
          try
          {
            sleep(dSleepSecs*1000);
          } // end try
          catch(InterruptedException anEx)
          {
          } // end catch
          vFolder.close(true);
        } // end while
        System.out.println("WatchDoc stopped.");
      } // end if
      else
      {
        System.err.println("Error: folder "+vFolder+" not found.");
        dStatus=3;
      } // end else
    } // end try
    catch(NoSuchProviderException anEx)
    {
      System.err.println(anEx);
      dStatus=1;
    } // end catch
    catch(MessagingException anEx)
    {
      System.err.println(anEx);
      dStatus=2;
    } // end catch
  } // end run
} // end class